% This script runs the simulation and experiment for the constrained LQR
% trajectory tracking method. The experiment is run by calling a LabView vi
% from within the script. 

clc
clear all
addpath('functions')

% Options
figbase  = 50;
verbose = 0;

controlParamName = 'exp01Controls.csv'; 
refTrajName      = 'ref_traj_track.csv';

controlDataRoot   = fullfile(pwd, 'data'); % base path for exp. data
modelPath         = pwd; % for simulink

% ---- Paths for shuffling data to labview and back. ------
% labview reads controller parameter data here:
controlDataPath   = fullfile(controlDataRoot, controlParamName);
% labview saves experimental results/data here:
dataOut_path = fullfile(controlDataRoot, 'exp01outputBOTH.csv');
% labview reads desired trajectory here:
refTrajPath       = fullfile(controlDataRoot, refTrajName);
% location of the vi which runs the experiment.
vipath            = 'C:\Users\arnold\Documents\labview\AFM_ss_singlematrix\play_AFMss_integral_trajTrack_extend.vi';

% ---------- Load Parametric Models  -----------
expName           = ['22-Jun-2016_exp01'];
modFitName    = [expName, '.mat'];
modFitPath    = fullfile(pwd, modFitName);

load(modFitPath, 'modelFit')
FitNum    = 'HighFidelity7';
%%
% 1). Design control gain, K.
% pstyle = '--k';
TOLbase   = 0.01;


% We will create three systems here:
% 1) sys_obs: This is the observer system for simulation. It only includes
%    Ns states, with NO delay, because we are implementing a reduced order
%    observer.
% 2) PLANT the S.S. system with delay absorbed as input delay
%    Use this to create the augmented system, (2).
% 3) sys_designK_aug: same as (1) but we augment with integral state.
%    Use this to design K.





%--------------------------------------------------------------------------
% Build models 


% 1). Create system with delay but without integral action

SYS = modelFit.(FitNum);
Nd  = SYS.InputDelay;
Ts  = SYS.Ts;
g1_inv = zpk([],[],1,Ts);

% 1)
sys_obs   = SYS;
sys_obs.InputDelay = 0;
Ns        = length(sys_obs.b);

% 2)
PLANT     = absorbDelay(SYS);

% 3). Augment system with integral action.
sys_designK_aug = addIntegral(PLANT);

% Create shift system output matrix;
C_delay      = zeros(1, Nd);
C_delay(end) = 1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Design compensator/gains.                                               %
% We create three gains here.                                             %
% 1). Estimator gain, in |R^{Ns}.                                         %
% 2) Tracking gain in |R^{Ns+Nd}                                          % 
% 3) inverse LQR gain, in |R^{Ns}. Only  used to generate (Q,R).          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% 1). Design estimator gains, L.
L_fdbk = fdbk(sys_obs, 'Q', 55.5*(sys_obs.c'*sys_obs.c), 'R', 1, 'obs',1);
L      = L_fdbk.K;
L_fdbk.notes = '55.5*C^TC';

% ---------------------------------------------------
% 2) Tracking gain.
p_int = 0.35;
gam_s = [1.3, 1.3, 1.0, 1.0, 1.0 1.0];
alp_s = [.7 .7 .7 .7 .7];

rhos  = [1.2, 1];
K_fdbk_trk = fdbk(sys_designK_aug, 'alps', alp_s, 'pint', p_int, 'rhos', rhos, 'gams', gam_s,...
               'doDelay', 1, 'rad', 0.4);
Ki_trk     = K_fdbk_trk.K(1);
K_trk      = K_fdbk_trk.K(2:end);

pz = 1;
if pz
    sys_CL = ss(sys_designK_aug.a-sys_designK_aug.b*K_fdbk_trk.K, sys_designK_aug.b,...
                sys_designK_aug.c, 0, Ts);
    F100 = figure(100);
    pzplot(sys_CL);
    zgrid
end  

% 3). LQR generation gain.        
% -------------------- Constrained LQR Stuff ------------------------------
%   These gains are only used to generate "fast" (Q,R) matrices for solving 
%   the CLQR problem.
gam_sLQR = [2.5, 2.5, 2.5, 1 1];
gam_sLQR = [1.5, 1.5, 1.5, 1 1];
p_intLQR = [1];
alp_sLQR = [0.95, 0.8, 0.8, 0.75, 0.7];
rho_sLQR = [1.23 1];

K_fdbkLQR = fdbk(sys_obs, 'alps', alp_sLQR, 'pint', p_intLQR, 'rhos', rho_sLQR,...
             'gams', gam_sLQR, 'doDelay', 1);
K_temp    = K_fdbkLQR.K;
%-----------------------------------------------------
[Q, R, KKK] = inverseLQR(sys_obs, K_temp);

if verbose >0
    sys_CL = ss(sys_obs.a-sys_obs.b*KKK, sys_obs.b,...
                sys_obs.c, 0, Ts);
    figure
    step(sys_CL)
end
err = norm(K_temp-KKK);
fprintf('----------------\nerror in K - lqr(Q,R): %f\n---------------\n', err);

Q = (Q'+Q)/2; % symmetrize, shoul be anyway. 
r = 1;
Qp = dare(sys_obs.a, sys_obs.b, Q, r); % QQ(end) is s.s. soln to dare.

umax = 3;

saveExpOn = 1;



% trun = Ts*N1;



% ------------------------------------------
% 1). Design the first leg.
N1    = 400;
ref_f_1 = 1.6;
ref_0_1 = 0;
TOL   = TOLbase*ref_f_1;

[uss_0, uss_f_1, x0_1, xf_1, xss_1] = yss2uss(sys_obs, ref_f_1, ref_0_1); 
x0_obs  = x0_1;

[~, ~, x0_full]   = yss2uss(PLANT, ref_f_1, ref_0_1);

x0_delay  = x0_full(Ns+1:end);
% x0_int    = uss_0 -ref_0*Nbar+K*x0_NOM;
x0_int = 0;
[H3, H2_x0, Ainq, binq] = clqrProblem(sys_obs, N1, Q, r, uss_f_1, umax, Qp);

slr = 1.5e3*Ts;
S = derMat(N1);
Ainq = [Ainq; S; -S];
binq = [binq; zeros(2*N1-2, 1)+slr];


u  = quadprog(H3, H2_x0*(x0_1-xss_1*uss_f_1), Ainq, binq)+uss_f_1;
uu1 = [u; u(end)];
% Simulate forward in time using a plant with delay included to acquire the
% optimal state and output trajectories. 
t1      = (0:1:N1)'*Ts;
[y1, t1, x1] = lsim(PLANT, uu1, t1, x0_full);

if 1
    figure(1); hold on
    plot(t1, uu1)
    title('u')
    figure(2); hold on
    plot(t1, y1); title('y')
end

%--------------------------------------------------------------------------
% 2. Design the SECOND leg.
ref_0_2 = ref_f_1;
ref_f_2 = -1.4;

% For the second leg, compute the whole 800 samples. Currently, the labview
% implementation which switches between reading the FIFO buffer and
% recycling the last element only works for the the first time, not the
% second time. 
N2 = 800;
[~,~,x0_2]    = yss2uss(sys_obs, ref_f_2, ref_0_2);
[uss_0_2, uss_f_2, x0_2, xf_2, xss_2] = yss2uss(sys_obs, ref_f_2, ref_0_2);
[~, ~, x0_full_2]   = yss2uss(PLANT, ref_f_2, ref_0_2);

[H3, H2_x0, Ainq, binq] = clqrProblem(sys_obs, N2, Q, r, uss_f_2, umax, Qp);
S = derMat(N2);
Ainq = [Ainq; S; -S];
binq = [binq; zeros(2*N2-2, 1)+slr];
Aeq  = zeros(1,N2); Aeq(1) = 1;
beq  = uss_0_2-uss_f_2

u  = quadprog(H3, H2_x0*(x0_2-xss_2*uss_f_2), Ainq, binq, Aeq, beq)+uss_f_2;
uu2 = [u; u(end)];
t2      = (0:1:N2)'*Ts;
[y2, t2, x2] = lsim(PLANT, uu2, t2, x0_full_2);
if 1
    figure(1); hold on
    plot(t2, uu2)
    title('u')
    figure(2); hold on
    plot(t2, y2); title('y')
end
%
y = [y1;y2];
x = [x1;x2];
uu = [uu1;uu2];
t = (0:1:length(y)-1)'*Ts;

%--------------------------------------------------------------------------
% For simulation, we extend the traj1 to be twice as long. This should not
% be done for the experimental data. 
% for simulation:
[y1_sim, x1_sim, uu1_sim, t1_sim] = extendTrajectory(PLANT, y1, x1, uu1,t1, uss_f_1, 2*length(t1));
y_sim  = [y1_sim;y2];
x_sim  = [x1_sim;x2];
uu_sim = [uu1_sim;uu2];
t_sim = (0:1:length(y_sim)-1)'*Ts;

uKx = uu_sim + x_sim*K_trk';
y_ref = timeseries(y_sim, t_sim);
uKx = timeseries(uKx, t_sim);

% For Experiment:
uKx_exp = uu + x*K_trk';
[~, ~, y_uKx]   = pack_uKx_y(uKx_exp, y, t);

% -------------------------------------------------------------------------
% -------------------------------------------------------------------------
% -----------------------Simulate and Plot---------------------------------
% 3) set sensor noise level. 
NoiseP = 0.0000000000;

THE_EXP_sim = [];
trun = y_ref.Time(end);
sim(fullfile(modelPath, 'AFM_SS_linearFDBK_obs_trackCLQR'))
u_track  = u_full; 
y_track  = yplant;

trackParams = struct('ref_f', ref_f_1, 'ref_0', ref_0_1,...
                     'ref_0_2', ref_f_2, 'ref_f_2', ref_f_2);
controller = struct('K_fdbkLQR', K_fdbkLQR, 'Q', Q, 'R', R,...
                    'K_trk', K_trk, 'Ki_trk', Ki_trk);
         
trackOpts = stepExpOpts('TOL', TOL, 'y_ref', ref_f_1, 'controller', controller, ...
                        'name', 'CLQR sim', 'pstyle', '--k', 'params', trackParams);

exp_track_sim = stepExp(y_track, u_track, trackOpts);

F_s = figure(figbase+2);
H1 = plot(exp_track_sim, F_s, 'textOn', 1);

%%

%----------------------------------------------------
% Save the controller to .csv file for implementation

% ------------------------------------------------------------------------
SettleTicks = 1000;

Iters = 1500;
pstyle = 'b';

exp_afm_exp = [];

Nbar = 0;
AllMatrix = packMatrix(sys_obs, L, K_trk);
saveControlData(AllMatrix, Ki_trk, Nbar, Ns, Nd, ref_f_1, y_uKx, controlDataPath, refTrajPath);

if exist('vi', 'var')
    clear vi; clear e;
end
[e, vi] = setupVI(vipath, 'SettleTicks', SettleTicks, 'Iters', Iters, ...
            'Stop', 0, 'Tab Control', 1, 'umax', umax, 'outputData BOTH', dataOut_path,...
            'reference traj path', refTrajPath, 'control_data_path', controlDataPath, 'N', length(t1));
vi.Run;

AFMdata = csvread(dataOut_path);
[y_expTrack, u_expTrack, xhat_expTrack, xdelay_expTrack] = unpackExpData(AFMdata, Ns, Ts);
         
expOpts = stepExpOpts(trackOpts, 'name', 'CLQR sim', 'pstyle', 'r',...
                      'params', trackParams)
exp_track_exp = stepExp(y_expTrack, u_expTrack, expOpts);
fname = sprintf('ref_f_1%.2f', ref_f_1);
fname = strrep(fname, '.', 'p');

exp_afm_exp = exp_track_exp;
ts_s = exp_track_exp.settle_time

Hs = plot(exp_track_exp, F_s, 'textOn', 1);
drawnow();

% calc settle times
sim_exp = exp_track_sim;
afm_exp = exp_afm_exp;
ref1 = ref_f_1
ref2 = ref_f_2;
TOL1 = abs(0.01*ref1);
TOL2 = abs(0.01*ref2);
ts1_sim = settle_time(sim_exp.y.Time(1:800), sim_exp.y.Data(1:800), ref1, TOL1);

ts1_afm = settle_time(afm_exp.y.Time(1:800), afm_exp.y.Data(1:800), ref1, TOL1);

t2_start = (length(t1)*2)+1;
ts2_sim = settle_time(sim_exp.y.Time(t2_start:end), sim_exp.y.Data(t2_start:end), ref2, TOL2)-Ts*t2_start;
ts2_afm = settle_time(afm_exp.y.Time(t2_start:end), afm_exp.y.Data(t2_start:end), ref2, TOL2)-Ts*t2_start;

%%
exp_ID = '05-18-2017_loopCLQR_trk2_extend_8um';
THE_EXP.systems.sys_designK_aug = sys_designK_aug;
THE_EXP.systems.sys_obs = sys_obs;
THE_EXP.systems.PLANT_NOM = PLANT;

THE_EXP.sims = exp_track_sim;
THE_EXP.afm  = exp_afm_exp;
THE_EXP.ts_s = struct('ts1_afm', ts1_afm, 'ts1_sim', ts1_sim,...
                      'ts2_afm', ts2_afm, 'ts2_sim', ts2_sim);

if saveExpOn
    saveExp(THE_EXP, pwd, exp_ID);
    
end




