% sysAug_integral = addIntegral(sys)
%
% Builds an augmented state space system that adds integral action to sys.
%
% Inputs: sys. A continuous or discrete time state-space system
%
% Outputs:
%   sysAug_integral. (PHI, GAM, H, 0) Has the form
%  For discrete time: 
%   PHI = | 1   C |
%         | 0   A |
%
%  For continuous time, PHI(1,1) = 0. (CP franklin, pg 587).
%  The rest are the same for both.
%
%   GAM = |0 |
%         |B |
%   
%   H   = |0  C |
%  
%   J = 0
% Thus, the new, augmented state has the form: x_aug = [x_int; x]^T
%
% Arnold Braker
% 6-10-2016


function sysAug_integral = addIntegral(sys)


if sys.InputDelay ~=0 || sys.OutputDelay ~= 0
    error('There should be no delay in the model sys that are not absorbed into extra states')
end

Ts      = sys.Ts;

Ns      = length(sys.b);
PHI_aug = [1, sys.c;
           zeros(Ns, 1), sys.a];
if Ts == 0
    PHI_aug(1,1) = 0;
end
       
GAM_aug = [0; sys.b];       
H_aug   = [0, sys.c];

sysAug_integral  = ss(PHI_aug, GAM_aug, H_aug, 0, Ts);



end