% [H3, H2_x0, Ainq, binq] = cqlrProblem(sys, N, Q, r,uss, umax, Qp)
%
% Condenses the constrained, discrete time, finite horizon LQR problem so it
% it can be solved by quadprog.
%
% ie, convert, min  sum (y(k) -ref)'q(y(k)-ref) + (u(k)-uss)'.r.(u(k)-uss) )
%                u
%               s.t. x(k+1) = Ax(k) + Bu(k)
% into 
%             
%             min  0.5*u(k)'.H3.u(k) + (x0'*H2_x0+H2_prim)'.u(k)
%              u
%             s.t.
%             Aeq*u(k) <= beq
%
% Inputs:
%  sys: discrete, state space dynamican SISO system.
%  N:   optimization horizon.
%  q, r, scaler weights for output (y(t)) control effort 
%  ref: reference input to track
%  umax: constraint on |u(t)| <= umax
%
%
% Outputs:
%  H3 -- jacobian N x N matrx
%  H2_x0 -- linear term, multiply by (x0_mpc-xss_mpc*uss),  N x Ns matrx.
%
% Example:
%
%   sys = ss(a, b, c, d, Ts)
%   q = 10;
%   r = 1;
%   N = 20;
%   
%   ref = 0.4; % steady state value of y, to track. 
%   uss = ref/(c*((eye(Ns)-a)\b));
% 
%   [H3, H2_x0, Aeq, beq] = clqrProblem(sys, N, q, r, ref, umax);
% 
%   H2 = H2_x0*(x0 - xss*uss)
% 
%   u = quadprog(H3, H2_x0*(x0_mpc-xss_mpc*uss), Ainq, binq)+uss;
 



function [H3, H2_x0, Ainq, binq] = cqlrProblem(sys, N, Q, r,uss, umax, Qp)
if sys.Ts == 0
    error('system "sys" should be discrete time dynamical system')
end

Ns = length(sys.b);

[a, b, c, d] = ssdata(sys);

% -------------------------------------------------------------------------
% Condense the LQR probelm
% X(k) = Mx(0) + H*u(k)
% Where G(i,:) = C*A^i
% and 


% make M;
% M = [A; A^2, A^2; ... A^N]'
M = []; CC = [];
for i = 1:N
    M = [M; a^i];
end

CC_1 = [eye(Ns); M(1:end-Ns, :)]*b;

for i=0:N-1
   CC = [CC, [zeros(Ns*i,1); CC_1(1:Ns*N-(i)*Ns)] ];
end

II = eye(N);
QQ = kron(II, Q);

QQ(N*Ns+1-Ns:end, N*Ns+1-Ns:end) = Qp;
RR = eye(N)*r;
H3 = 2*(CC'*QQ*CC + RR);

H3 = (H3+H3')/2; %Symmetrize, should be anyway...
H2_x0 = (2*M'*QQ*CC)';
Ainq = [eye(N); -eye(N)];


ubp = umax - uss;
ubm = -umax - uss;
binq = [ones(N, 1)*ubp; -ones(N, 1)*ubm];
% binq = [binq; ];

end