% Given a dynamical system, Gz, creates a an object of first and second
% order transfer functions, spilt into complex zeros (cz), complex poles
% (cp) real zeros (rz), real poles (rp), delay (del), a gain, K. Each of,
% e.g. cz, is an array of complex zero transfer functions sorted in
% ascending frequency of the mode.
%
% Properties
%         cz;
%         cp;
%         rz;
%         rp;
%         del;
%         K;
%         Ts;
%
%
% See also:
%   decimated2TF.m
%   pairDecimated.m


classdef decimateTF
    properties
        cz;
        cp;
        rz;
        rp;
        del;
        K;
        Ts;
    end
    
    methods
        function obj = decimateTF(Gz)
        Gz = zpk(Gz);
        Ts = Gz.Ts;

        [z, p, k] = zpkdata(absorbDelay(Gz), 'v');
        
        kz_real  = find( imag(z)==0);
        kz_cmplx = find( imag(z)~=0);

        kp_real  = find( imag(p)==0 & real(p)~=0);
        kd       = find( imag(p)==0 & real(p)==0);
        kp_cmplx = find( imag(p)~=0);



        % complex zero
        cz.G = []; cz.wn = [];
        cp.G = []; cp.wn = [];
        rz.G = []; rz.wn = [];
        rp.G = []; rp.wn = [];
        for i = 1:2:length(kz_cmplx)
           cz.G     = [cz.G; zpk([z(kz_cmplx(i)), z(kz_cmplx(i+1))], [], 1, Ts)];
            cz.wn   = [cz.wn; abs(log(z(kz_cmplx(i)))/Ts)];
        end
        
        % complex poles
        for i = 1:2:length(kp_cmplx)
           cp.G = [cp.G; zpk([], [p(kp_cmplx(i)), p(kp_cmplx(i+1))], 1, Ts)];
           cp.wn = [cp.wn; abs(log(p(kp_cmplx(i)))/Ts)];
        end

        % real zeros
        for i = 1:length(kz_real)
            rz.G = [rz.G; zpk(z(kz_real(i)), [], 1, Ts)];
            rz.wn = [rz.wn; abs(log(z(kz_real(i)))/Ts)];
        end

        % real pole
        for i=1:length(kp_real)
            rp.G = [rp.G; zpk([], p(kp_real(i)), 1, Ts)];
            rp.wn = [rp.wn; abs(log(p(kp_real(i)))/Ts)];
        end

        obj.cz = cz;
        obj.cp = cp;
        obj.rz = rz;
        obj.rp = rp;
        
        
        % Sort in ascending order of wn.
        [obj.cz.wn, I]  = sort(obj.cz.wn);
        obj.cz.G       = obj.cz.G(I);
        
        % Sort in ascending order of wn.
        [obj.cp.wn, I]  = sort(obj.cp.wn);
        obj.cp.G       = obj.cp.G(I);
        
        % Sort in ascending order of wn.
        [obj.rp.wn, I]  = sort(obj.rp.wn);
        obj.rp.G       = obj.rp.G(I);
        
        % Sort in ascending order of wn.
        [obj.rz.wn, I]  = sort(obj.rz.wn);
        obj.rz.G       = obj.rz.G(I);
        
        
        
        % Delay
        if kd
            obj.del = zpk([], zeros(1, length(kd)), 1, Ts);
        else
            obj.del = zpk([], [], 1, Ts);
        end
        
        % pure gain
        obj.K = zpk([], [], k, Ts);



        end
        
        function H = deleteModes(obj, dlt)
            % H = deleteModes(obj, dlt)
            % dlt.cz, dlt.rz, dlt.rp, dlt.cp contain
            % indeces of modes to delete.
            fname_s = fieldnames(dlt);
            for i=1:length(fname_s)
                ind = dlt.(fname_s{i});
                try
                obj.(fname_s{i}).G(ind) = []; 
                obj.(fname_s{i}).wn(ind) = []; 
                catch
                    keyboard
                end
            end
            H = obj;
        end
        function SYS = ss(obj)
           SYS = ss(obj.K);
           obj = equalizeModes(obj);
           
           for i=1:length(obj.cp.G)
               if i <= length(obj.cz.G)
                   SYS = SYS*ss(obj.cz.G(i)*obj.cp.G(i));
               else
                   SYS = SYS*ss(obj.cp.G(i));
               end
           end
          for i=1:length(obj.rp.G)
               if i <= length(obj.rz.G)
                   SYS = SYS*ss(obj.rz.G(i)*obj.rp.G(i));
               else
                   SYS = SYS*ss(obj.rp.G(i));
               end
          end
           SYS = minreal(SYS);
         end
        
         function hand = bode(obj, f_s, F1)
            SYS = ss(obj);
            if ~exist('F1', 'var')
                F1 = figure(100);
            end
            sysFRF = getFRF(SYS, f_s*2*pi);
            frfBode(sysFRF, f_s, F1, 'g', 'Hz');
            [pole_freq_complex_s, pole_freq_real_s, ...
           zero_freq_complex_s, zero_freq_real_s,...
           hands, legend_handles] = plotPZ_freqs(SYS, F1, 0, 0);
       
%              keyboard
             for i=1:length(obj.cp.wn)
                 w_i = obj.cp.wn(i);
                 f_i = w_i/2/pi;
                 mag_i = 20*log10( bode(SYS, w_i));
                 tx_i = sprintf('p%d', i);
                 text(f_i, mag_i, tx_i);
             end
             for i=1:length(obj.cz.wn)
                 w_i = obj.cz.wn(i);
                 f_i = w_i/2/pi;
                 mag_i = 20*log10( bode(SYS, w_i));
                 tx_i = sprintf('z%d', i);
                 text(f_i, mag_i, tx_i);
             end
              for i=1:length(obj.rz.wn)
                 w_i = obj.rz.wn(i);
                 f_i = w_i/2/pi;
                 mag_i = 20*log10(bode(SYS, w_i));
                 tx_i = sprintf('z%d', i);
                 text(f_i, mag_i, tx_i);
              end
              for i=1:length(obj.rp.wn)
                 w_i = obj.rp.wn(i);
                 f_i = w_i/2/pi;
                 mag_i = 20*log10(bode(SYS, w_i));
                 tx_i = sprintf('z%d', i);
                 text(f_i, mag_i, tx_i);
             end
         end
        
        
    end

    
    
    
    
    
    
end

function H = equalizeModes(H)
    nrz = length(H.rz.G);
    nrp = length(H.rp.G);
    ncz = length(H.cz.G);
    ncp = length(H.cp.G);
    g = zpk([],[], 1, H.Ts);
    if nrz < nrp
        h(1:nrp-nrz) = g;
        H.rz.G = [H.rz.G; h];
    elseif nrz > nrp
        h(1:nrz-nrp) = g;
        H.rp.G = [H.rp.G; h];
    end
    if ncz < ncp
        h(1:ncp-ncz) = g;
        H.cz.G = [H.cz.G; h];
    elseif ncz > ncp
        h(1:ncz-ncp) = g;
        H.cp.G = [H.cp.G; h];
    end
    
    
    
end
% function plotText(SYS, 













