% [h_mag, h_phase] = frfBode(Gz, w_s, F1, plotStyle, freqUnit)
%
% Gz can be an LTI system, or complex frequency response data in the form of
% Gz_frf(w_s) = |mags|.*exp(j*phase). 
%
% IF Gz is an LTI system, then freqUnit is required. If freqUnit starts
% with (H|h), assume hertz, and convert to radians while computing the frf.
% If 
%
%   Inputs:
%       Gz        : Either complex FRF data, or an LTI dynamical system.
%       w_s       : Frequencies to plot at.
%       F1        : Figure handle to plot to. (optional)
%       plotStyle : A string defining the plot style, e.g. '--g'
%       freqUnit  : A string defining the frequency unit. This input is
%                   required if Gz is an LTI system. freqUnit must start 
%                   with either (h|H) for hertz or (r|R) for radians/sec.
%                   (This is needed so the frf of Gz can be correctly computed).
%
%   Outputs:
%   h_mag, h_phase: plot handles to the magnitude and phase plots
%   respectively.
%
%
% Arnold Braker
% May 6 2016
%
% Modified:
% RAB. 6-10-2016
% RAB. 6-18-2016

function [h_mag, h_phase] = frfBode(Gz, w_s, F1, plotStyle, freqUnit)


if ~exist('F1', 'var') || isempty(F1)
    F1 = figure;
end

if ~exist('plotStyle', 'var') || isempty(plotStyle)
    plotStyle = 'b';
end

if isa(Gz, 'lti')
    if freqUnit(1) == 'r' || freqUnit(1) =='R'
        Gz_frf = getFRF(Gz, w_s);
    elseif freqUnit(1) == 'H' || freqUnit(1) == 'h'
        Gz_frf = getFRF(Gz, w_s*2*pi);
    else
        error('Unkown frequency unit, freqUnit. Should start with (h|H) or (r|R)');
    end
else
    Gz_frf = Gz;
end


magsDB = 20*log10(abs(Gz_frf));
phase  = unwrap(angle(Gz_frf))*180/pi;

figure(F1)
subplot(2,1,1)
    [h_mag] = semilogx(w_s, magsDB, plotStyle, 'LineWidth', 1.5);
    grid on; hold on;
    xlim([w_s(1), w_s(end)]);
    ylabel('Mag [dB]')
subplot(2,1,2)
    h_phase = semilogx(w_s, phase, plotStyle, 'LineWidth', 1.5);
    grid on; hold on;
    xlim([w_s(1), w_s(end)]);
    ylabel('Phase [deg]')
if exist('freqUnit', 'var')
    xlabel(freqUnit)
end


end