% [pole_freq_complex_s, pole_freq_real_s, ...
%          zero_freq_complex_s, zero_freq_real_s,...
%          hands, legend_handles] = plotPZ_freqs(Gz_TF, F1, linesOn,
%          freqFlag, ax
% Plots lines and markers of the discrete transfer function, Gz_TF of all
% poles and zeros to the supplied plot handle, F1
% Inputs: 
%   Gz_TF: discrete time dynamical system.
%   F1:    Figure handle to plot to.
%   linesOn: flag, 0--> no lines, 1--> lines. Default is 0.
%   freqFlag: 0--> Hz, 1--> radians. Default is 0, ie hz.
%   ax:       (optional) the axis to plot to. If not supplied, default is
%             to plot to subplot(2,1,1)
%
%
%
% Outputs:
% Returns the real and complex pole frequencies, independantly, in Hz
% and real and complex zero frequencies. 
% 
% Also returns a structure, hands, containing plot handles of all plots
% with the following format:
%
% hands.prl   - pole real line
% hands.prm   - pole real marker
% hands.pcl   - pole complex line
% hands.pcm   - pole complex marker
%
% hands.zrl   - zero real line
% hands.zrm   - zero real marker
% hands.zcl   - zero complex line
% hands.zcm   - zero complex marker


function [pole_freq_complex_s, pole_freq_real_s, ...
          zero_freq_complex_s, zero_freq_real_s,...
          hands, legend_handles] = plotPZ_freqs(Gz_TF, F1, linesOn, freqFlag, ...
                                                ax)
      if ~exist('freqFlag', 'var')
          freqFlag = 0; % Hz.
      end
      if ~exist('linesOn', 'var')
          linesOn = 0;
      end
      
      pole_freq_complex_s = [];
      pole_freq_real_s    = [];
      zero_freq_real_s    = [];
      zero_freq_complex_s = [];
      if nargin <3
          linesOff = 0
      end
      
    Ts = Gz_TF.Ts;
    [mags, phases, ws] = bode(Gz_TF);
    mags = 20*log10(squeeze(mags));
    phases = squeeze(phases);
    
    if ~freqFlag
        freq_mult = 2*pi;
    else
        freq_mult = 1;
    end
    fs = ws/freq_mult;
    % Pole Frequencies
    p_locs_z = pole(Gz_TF);
    % Convert to S-plane location
    p_locs   = log(p_locs_z)/Ts;
    kr = 1;
    kc = 1;
    skip = 0;
    
    % Separate Poles with purely real parts from
    % Complex Conjugate Poles
    P_real = [];
    P_complex = [];
    for i = 1:length(p_locs)
       if (imag(p_locs_z(i)) == 0)
           p_real(kr) = abs(p_locs(i) );
           kr = kr +1;
       elseif (skip == 0) 
           P_complex(kc) = p_locs(i);
           kc = kc + 1;
           skip = 1;
       elseif (skip == 1)
           skip = 0;
       end
    end
   % Set Figure.
    figure(F1)
    if ~exist('ax', 'var')
        subplot(2,1,1)
    else
        axes(ax);
    end
    ylm = ylim;
    legend_handles = [];
    % Real Poles
    if exist('p_real', 'var') >= 1
        pole_freq_real_s    = abs(p_real)/freq_mult;
        for j = 1:length(pole_freq_real_s)
            f_j = pole_freq_real_s(j);
            if linesOn == 1
                hands.prl(j) = plot([f_j, f_j], ylm, '--k');
            end
            hands.prm(j) = plot(f_j, interp1(fs, mags, f_j),'xk');
        end
        set(hands.prm, 'DisplayName', 'Real Poles');
        legend_handles = [legend_handles, hands.prm(1)];
    end

    % Complex Poles
    if length(P_complex) >= 1
        pole_freq_complex_s = abs(P_complex)/freq_mult;
        for i = 1:length(pole_freq_complex_s)
            f_i = pole_freq_complex_s(i);
            if linesOn == 1
                hands.pcl(i) = plot([f_i, f_i], ylm, '-k');
            end
            hands.pcm(i) = plot(f_i, interp1(fs, mags, f_i),'*k');
        end
        set(hands.pcm, 'DisplayName', 'Complex Poles')
        legend_handles = [legend_handles, hands.pcm(1)];
    end
    
  % -----------------------------------------------------------------------
  % --------------------- Now The zeros -----------------------------------
  
    % Zero Frequencies
    z_locs_z = zero(Gz_TF);
    z_locs   = log(z_locs_z)/Ts;
    kr = 1;
    kc = 1;
    skip = 0;
    
    % Separate Poles with purly real parts from
    % Complex Conjugate Poles
    z_real    = [];
    z_complex = [];
    for i = 1:length(z_locs)
       if (imag(z_locs_z(i)) == 0)
           z_real(kr) = abs(z_locs(i) );
           kr = kr +1;
       elseif (skip == 0) 
           z_complex(kc) = z_locs(i);
           kc = kc + 1;
           skip = 1;
       elseif (skip == 1)
           skip = 0;
       end
    end
    
    
    
    if length(z_real) >= 1
        zero_freq_real_s    = abs(z_real)/freq_mult;
        for j = 1:length(zero_freq_real_s)
            f_j = zero_freq_real_s(j);
            if linesOn == 1
                hands.zrl(j) = plot([f_j, f_j], ylm, '--m');
            end
            hands.zrm(j) = plot(f_j, interp1(fs, mags, f_j),'sm');
        end
        set(hands.zrm, 'DisplayName', 'Real Zero');
        legend_handles = [legend_handles, hands.zrm(1)];
    end
%     keyboard
    if length(z_complex) >= 1
%         keyboard
        zero_freq_complex_s = abs(z_complex)/freq_mult;
        for i = 1:length(zero_freq_complex_s)
            f_i = zero_freq_complex_s(i);
            if linesOn == 1
                hands.zcl(i) = plot([f_i, f_i], ylm, '-m');
            end
            hands.zcm(i) = plot(f_i, interp1(fs, mags, f_i),'om');
        end
    set(hands.zcm, 'DisplayName', 'Complex Zeros');
    legend_handles = [legend_handles, hands.zcm(1)];
    end    

    


end